"use strict";

require("core-js/modules/es.symbol");

require("core-js/modules/es.array.concat");

require("core-js/modules/es.array.filter");

require("core-js/modules/es.array.for-each");

require("core-js/modules/es.object.define-property");

require("core-js/modules/es.object.get-own-property-descriptor");

require("core-js/modules/es.object.keys");

require("core-js/modules/es.object.to-string");

require("core-js/modules/es.promise");

require("core-js/modules/web.dom-collections.for-each");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withBakedInitialProps = void 0;

require("regenerator-runtime/runtime");

var _env = require("./utils/env");

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var notImplemented = function notImplemented() {};
/**
 * @description
 * Callback wich describes when the data should be baked.
 * @callback bakeWhen
 *
 * @param {boolean} defaultValue
 * @param {Object} ctx
 * @returns {boolean}
 */


var bakeWhen = function bakeWhen(defaultValue, ctx) {
  return defaultValue;
};
/**
 * @description
 * CallBack which describes when the intiial initialProps of the PageComponent should be used instead of the baked data.
 * @param {boolean} defaultValue
 * @param {Object} ctx
 * @returns {boolean}
 */


var getInitialPropsWhen = function getInitialPropsWhen(defaultValue, ctx) {
  return defaultValue;
};
/**
 * @description
 * The base HOC which can be used to implement your own bake storage
 *
 * @param {React.Component} PageComponent
 * @param {Object} options
 * @param {bakeWhen} options.bakeWhen
 * @param {getInitialPropsWhen} options.getInitialPropsWhen
 * @returns {React.Component}
 */


var withBakedInitialProps = function withBakedInitialProps(PageComponent) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var opts = _objectSpread({
    bakeWhen: bakeWhen,
    getInitialPropsWhen: getInitialPropsWhen
  }, options);

  var BakedInitialPropsPage = function BakedInitialPropsPage(props) {
    return _react["default"].createElement(PageComponent, props);
  };

  BakedInitialPropsPage.getInitialProps =
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee() {
    var _len,
        params,
        _key,
        ctx,
        getBakedInitialProps,
        bakeInitialProps,
        defaultGetInitialPropsWhen,
        defaultBakeWhen,
        data,
        _args = arguments;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            for (_len = _args.length, params = new Array(_len), _key = 0; _key < _len; _key++) {
              params[_key] = _args[_key];
            }

            ctx = params[0];
            getBakedInitialProps = PageComponent.getBakedInitialProps || notImplemented;
            bakeInitialProps = PageComponent.bakeInitialProps || notImplemented;

            defaultGetInitialPropsWhen = function defaultGetInitialPropsWhen() {
              return !(0, _env.isExported)(ctx);
            };

            defaultBakeWhen = function defaultBakeWhen() {
              return (0, _env.onExport)(ctx);
            };

            if (!(opts.bakeWhen(defaultBakeWhen(ctx), ctx) || opts.getInitialPropsWhen(defaultGetInitialPropsWhen(ctx), ctx))) {
              _context.next = 14;
              break;
            }

            _context.next = 9;
            return PageComponent.getInitialProps.apply(PageComponent, params);

          case 9:
            data = _context.sent;

            if (!opts.bakeWhen(defaultBakeWhen(ctx), ctx)) {
              _context.next = 13;
              break;
            }

            _context.next = 13;
            return bakeInitialProps(data, ctx);

          case 13:
            return _context.abrupt("return", data);

          case 14:
            _context.next = 16;
            return getBakedInitialProps.apply(void 0, params);

          case 16:
            return _context.abrupt("return", _context.sent);

          case 17:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return BakedInitialPropsPage;
};

exports.withBakedInitialProps = withBakedInitialProps;